@@ -1,8 +1,8 @@
-(* 
- * ExtInt32 - Extended 32-bit integers
+(*
+ * BatInt32 - Extended 32-bit integers
  * Copyright (C) 2007 Bluestorm <bluestorm dot dylc on-the-server gmail dot com>
  *               2008 David Teller
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -23,15 +23,22 @@
 open BatNumber
 
 let (|>) x f = f x
-      
+
 let to_byte n = Int32.logand 0xffl n |> Int32.to_int |> Char.chr
 let of_byte b = Char.code b |> Int32.of_int
-    
+
+(*Q to_byte, of_byte
+  Q.char (fun c -> Pervasives.(=) (to_byte (of_byte c)) c)
+ *)
+
+(*$T to_byte
+  to_byte 256l = to_byte 0l
+ *)
+
 (* really need to just blit an int32 word into a string and vice versa *)
-    
-let pack str pos item = 
-  if String.length str > pos + 4 then failwith "Int32.pack: pos + 4 not within string";
-  if pos < 0 then failwith "Int32.pack: pos negative";
+let pack str pos item =
+  if String.length str > pos + 4 then invalid_arg "Int32.pack: pos too close to end of string";
+  if pos < 0 then invalid_arg "Int32.pack: pos negative";
   str.[pos] <- to_byte item;
   let item = Int32.shift_right item 8 in
   str.[pos+1] <- to_byte item;
@@ -39,9 +46,9 @@ let pack str pos item =
   str.[pos+2] <- to_byte item;
   let item = Int32.shift_right item 8 in
   str.[pos+3] <- to_byte item (* optimize out last logand? *)
-    
-let pack_big str pos item = 
-  if String.length str > pos + 4 then failwith "Int32.pack_big: pos + 4 not within string";
+
+let pack_big str pos item =
+  if String.length str > pos + 4 then failwith "Int32.pack_big: pos too close to end of string";
   if pos < 0 then failwith "Int32.pack_big: pos negative";
   str.[pos+3] <- to_byte item;
   let item = Int32.shift_right item 8 in
@@ -50,35 +57,33 @@ let pack_big str pos item =
   str.[pos+1] <- to_byte item;
   let item = Int32.shift_right item 8 in
   str.[pos] <- to_byte item (* optimize out last logand? *)
-    
-let unpack str pos = 
+
+let unpack str pos =
   if String.length str > pos + 4 then failwith "Int32.unpack: pos + 4 not within string";
   if pos < 0 then failwith "Int32.unpack: pos negative";
-  let shift n = Int32.shift_left n 8 
+  let shift n = Int32.shift_left n 8
   and add b n = Int32.add (of_byte b) n in
-  of_byte str.[pos+3] |> shift |> add str.[pos+2] |> shift 
+  of_byte str.[pos+3] |> shift |> add str.[pos+2] |> shift
     |> add str.[pos+1] |> shift |> add str.[pos]
   (* TODO: improve performance of bit twiddling?  will these curried functions get inlined? *)
 
-let unpack_big str pos = 
+let unpack_big str pos =
   if String.length str > pos + 4 then failwith "Int32.unpack: pos + 4 not within string";
   if pos < 0 then failwith "Int32.unpack: pos negative";
-  let shift n = Int32.shift_left n 8 
+  let shift n = Int32.shift_left n 8
   and add b n = Int32.add (of_byte b) n in
-  of_byte str.[pos] |> shift |> add str.[pos+1] |> shift 
+  of_byte str.[pos] |> shift |> add str.[pos+1] |> shift
     |> add str.[pos+2] |> shift |> add str.[pos+3]
-	
+
 
 module BaseInt32 = struct
   include Int32
-    
+
   let modulo = rem
   let pow = generic_pow ~zero ~one ~div_two:(fun n -> shift_right n 1) ~mod_two:(logand one) ~mul:mul
 end
 
 include BatNumber.MakeNumeric(BaseInt32)
-module Infix = BatNumber.MakeInfix(BaseInt32)
-module Compare = BatNumber.MakeCompare(BaseInt32)
 
 let min_int = Int32.min_int
 let max_int = Int32.max_int
@@ -116,5 +121,5 @@ external format : string -> int32 -> string = "caml_int32_format"
 type bounded = t
 let min_num, max_num = min_int, max_int
 
-let print out t = BatInnerIO.Printf.fprintf out "%lx" t
-let t_printer paren out t = print out t
+let print out t = BatInnerIO.nwrite out (to_string t)
+let print_hex out t = BatPrintf.fprintf out "%lx" t
