@@ -1,7 +1,7 @@
-(* 
+(*
  * LazyListLabels - lazily-computed lists
  * Copyright (C) 2008 David Teller
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -25,11 +25,11 @@ exception Empty_list
 exception Invalid_index of int
 exception Different_list_size of string
 
-  
+
 (** {6  Types} *)
 type 'a node_t =
-  | Nil 
-  | Cons of 'a * 'a t 
+  | Nil
+  | Cons of 'a * 'a t
 and 'a t =
   ('a node_t) Lazy.t
 
@@ -62,14 +62,14 @@ let from_while f =
       | None   -> Nil
       | Some x -> Cons (x, aux ()) ) in aux ()
 
-let from f = 
+let from f =
   let f' () =
     try  Some (f ())
     with No_more_elements -> None
   in from_while f'
 
 let seq data next cond =
-  let rec aux data = 
+  let rec aux data =
     if cond data then Cons (data, lazy (aux (next data)))
     else              Nil
   in lazy (aux data)
@@ -88,9 +88,9 @@ let from_loop (data:'b) (next:'b -> ('a * 'b)) : 'a t=
     with No_more_elements -> None
   in unfold data f'
 
-let init n f = 
-  let rec aux i = 
-    if i < n then lazy (Cons (f i, aux ( i + 1 ) ) )  
+let init n f =
+  let rec aux i =
+    if i < n then lazy (Cons (f i, aux ( i + 1 ) ) )
     else          nil
   in if n < 0 then raise (Invalid_argument "LazyList.init")
      else          aux 0
@@ -107,23 +107,23 @@ let make n x =
 *)
 
 let iter f l =
-  let rec aux l = match next l with 
-    | Cons (x, t) -> (f x; aux t) 
+  let rec aux l = match next l with
+    | Cons (x, t) -> (f x; aux t)
     | _ -> ()
   in aux l
-  
+
 let iteri f l =
-  let rec aux i l = match next l with 
-    | Cons (x, t) -> (f i x; aux (i + 1) t) 
+  let rec aux i l = match next l with
+    | Cons (x, t) -> (f i x; aux (i + 1) t)
     | _ -> ()
   in aux 0 l
-  
+
 let map f l =
   let rec aux rest =  match next rest with
     | Cons (x, (t : 'a t)) -> Cons (f x, lazy (aux t))
     | Nil                  -> Nil
   in lazy (aux l)
-  
+
 let mapi f l =
   let rec aux rest i =
     match next rest with
@@ -133,11 +133,11 @@ let mapi f l =
 
 let fold_left f init l =
   let rec aux acc rest =
-    match next rest with 
-      | Cons (x, t) -> aux (f acc x) t 
+    match next rest with
+      | Cons (x, t) -> aux (f acc x) t
       | _           -> acc
   in aux init l
-  
+
 let fold_right f init l =
   let rec aux rest =
     match next rest with | Cons (x, t) -> f x (aux t) | _ -> init
@@ -150,7 +150,7 @@ let may_find p l =
     match next l with
       | Nil         -> None
       | Cons (x, t) -> if p x then Some x else aux t
-  in aux l 
+  in aux l
 
 let may_rfind p l =
   let rec aux l acc =
@@ -210,10 +210,10 @@ let rindex_ofq e l =
     | None        -> None
     | Some (i, _) -> Some i
 
-  
+
 (** {6  Common functions}*)
 let length l = fold_left (fun n _ -> n + 1) 0 l
-  
+
 let is_empty l = match next l with
   | Nil -> true
   | _   -> false
@@ -229,7 +229,7 @@ let hd list =
   match next list with
   | Cons (x, _) -> x
   | _ -> raise Empty_list
-  
+
 let first = hd
 
 
@@ -245,8 +245,8 @@ let tl list =
   match next list with
   | Cons (_, t) -> t
   | _ -> raise Empty_list
-  
-  
+
+
 let at list n =
   let rec aux list i =
     match ((next list), i) with
@@ -258,7 +258,7 @@ let at list n =
 let nth = at
 
 let rev list = fold_left (fun acc x -> Lazy.lazy_from_val (Cons (x, acc))) nil list
-  
+
 (**Revert a list, convert it to a lazy list.
    Used as an optimisation.*)
 let rev_of_list (list:'a list) = List.fold_left (fun acc x -> Lazy.lazy_from_val (Cons (x, acc))) nil list
@@ -269,7 +269,7 @@ let eager_append (l1 : 'a t) (l2 : 'a t) =
     | Cons (x, t) -> cons x (aux t)
     | _           -> l2
   in aux l1
-  
+
 let rev_append (l1 : 'a t) (l2 : 'a t) =
   let rec aux list acc =
     match next list with
@@ -290,14 +290,14 @@ let append (l1 : 'a t) (l2 : 'a t) =
       | Cons (x, (t : 'a t)) -> Cons (x, lazy (aux t))
       | _                    -> Lazy.force l2
   in lazy (aux l1)
-  
+
 let ( ^@^ ) = append
-  
+
 let flatten (lol : ('a t) list) =
   ListLabels.fold_left ~init: nil ~f: append lol
-  
+
 let concat  (lol : ('a t) t) = fold_left append nil lol
-  
+
 (**
    {6  Conversions}
 *)
@@ -305,7 +305,7 @@ let concat  (lol : ('a t) t) = fold_left append nil lol
    Eager conversion to string.
 *)
 let to_list l = fold_right (fun x acc -> x :: acc) [] l
-  
+
 (**
    Lazy conversion to stream.
 *)
@@ -315,14 +315,14 @@ let to_stream l =
     | Cons (x, t) -> Stream.icons x (Stream.slazy (fun _ -> aux t))
     | _ -> Stream.sempty
   in aux l
-  
+
 (**
    Eager conversion to array.
 *)
 let to_array l = Array.of_list (to_list l)
 
 let enum l =
-  let rec aux l = 
+  let rec aux l =
     let reference = ref l in
       BatEnum.make ~next:(fun () -> match next !reference with
 			 | Cons(x,t) -> reference := t; x
@@ -330,21 +330,21 @@ let enum l =
         ~count:(fun () -> length !reference)
         ~clone:(fun () -> aux !reference)
   in aux l
-  
+
 (**
    Lazy conversion from lists
 
-   Albeit slower than eager conversion, this is the default mechanism for converting from regular 
+   Albeit slower than eager conversion, this is the default mechanism for converting from regular
    lists to lazy lists.  This for two reasons :
    * if you're using lazy lists, total speed probably isn't as much an issue as start-up speed
    * this will let you convert regular infinite lists to lazy lists.
 *)
 let of_list l =
   let rec aux = function
-    | []     -> nil 
+    | []     -> nil
     | h :: t -> lazy (Cons (h, aux t))
   in aux l
-  
+
 (**
    Lazy conversion from stream.
 *)
@@ -356,13 +356,13 @@ let of_stream s =
       | Some h -> (Stream.junk __strm; lazy (Cons (h, aux s)))
       | _ -> nil
   in aux s
-  
+
 (**
    Eager conversion from lists
 *)
 let eager_of_list l =
   ListLabels.fold_right ~init: nil ~f: (fun x acc -> Lazy.lazy_from_val (Cons (x, acc))) l
-  
+
 (**
    Eager conversion from array
 *)
@@ -375,15 +375,15 @@ let of_array l =
 let of_enum e =
   let rec aux () =
     lazy (match BatEnum.get e with
-      |	Some x -> Cons (x, aux () ) 
+      |	Some x -> Cons (x, aux () )
       | _      -> Nil )
   in
     aux ()
-  
+
 (**
    {6  Predicates}
 *)
-let filter f l = 
+let filter f l =
   let rec next_true l = match next l with (*Compute the next accepted predicate without thunkification*)
     | Cons (x, l) when not (f x) -> next_true l
     | l                          -> l
@@ -393,9 +393,9 @@ let filter f l =
     | Nil         -> Nil)
   in aux l
 
-let filter_map f l = 
+let filter_map f l =
   let rec next_true l = match next l with (*Compute the next accepted predicate without thunkification*)
-    | Cons (x, l) -> 
+    | Cons (x, l) ->
 	begin
 	  match f x with
 	    | Some v  -> Some (v, l)
@@ -414,28 +414,28 @@ let filter_map f l =
     | Cons (_, t)          -> aux t
     | Nil                  -> Nil
   in lazy (aux l)*)
-  
+
 let exists f l =
   let rec aux rest = match next rest with
-    | Cons (x, t) when f x -> true
+    | Cons (x, _) when f x -> true
     | Cons (_, t)          -> aux t
     | Nil                  -> false
   in aux l
-  
+
 let for_all f l =
   let rec aux rest = match next rest with
     | Cons (x, t) when f x -> aux t
-    | Cons (_, t)          -> false
+    | Cons _               -> false
     | Nil                  -> true
   in aux l
-  
+
 let range a b =
   let rec increasing lo hi =
     if lo > hi then nil else lazy (Cons (lo, increasing (lo + 1) hi))
   in
-    (*      and     decreasing lo hi = if lo > hi then 
+    (*      and     decreasing lo hi = if lo > hi then
 	nil
-      else 
+      else
 	lazy (Cons hi (decreasing lo (hi - 1)))*)
     if b >= a then increasing a b else (*decreasing b a*) nil
 
@@ -450,7 +450,7 @@ let split_at n l =
 let split_nth = split_at
 
 let mem   e = exists (( = ) e)
- 
+
 let memq  e = exists (( == ) e )
 
 let assoc e l = snd (find (fun (a,_) -> a = e) l)
@@ -465,17 +465,17 @@ let mem_assq e l = BatOption.is_some (may_find (fun (a, _) -> a == e) l)
 
 (*  let rec aux rest = match next rest with
     | Cons (h, t) ->
-        (match f h with 
+        (match f h with
 	   | None   -> lazy (aux t)
 	   | Some x -> cons x (lazy (aux t)))
     | Nil -> Nil
   in lazy (aux l)*)
 
 let unique ?(cmp = compare) l =
-  let set      = ref (BatPMap.create cmp) in
-  let should_keep x = 
-    if BatPMap.mem x !set then false
-    else ( set := BatPMap.add x true !set; true )
+  let set      = ref (BatMap.PMap.create cmp) in
+  let should_keep x =
+    if BatMap.PMap.mem x !set then false
+    else ( set := BatMap.PMap.add x true !set; true )
   in
   (* use a stateful filter to remove duplicate elements *)
   filter should_keep l
@@ -510,10 +510,10 @@ let remove_all x l =
 
 (** An infinite list of nothing *)
 let rec eternity = lazy (Cons ((), eternity))
-  
+
 let take n l = fst (split_at n l)
 
-let drop n l = 
+let drop n l =
   let rec aux l i =
     if i = 0 then l
     else match next l with
@@ -534,7 +534,7 @@ let take_while p =
     | _                  -> rev_of_list acc
   in aux []
 
-let sort ?cmp l = of_list (BatList.sort ?cmp (to_list l))
+let sort ?(cmp=Pervasives.compare) l = of_list (List.sort cmp (to_list l))
 
 let stable_sort cmp l = of_list (List.stable_sort cmp (to_list l))
 
@@ -554,8 +554,8 @@ let iter2 f l1 l2 =
 	| _                             -> raise (Different_list_size "LazyList.iter2")
   in aux l1 l2
 
-let fold_left2 f acc l1 l2 = 
-  let rec aux acc l1 l2 = 
+let fold_left2 f acc l1 l2 =
+  let rec aux acc l1 l2 =
       match (next l1, next l2) with
 	| (Cons (h1, t1), Cons(h2, t2)) -> aux (f acc h1 h2) t1 t2
 	| (Nil, Nil)                    -> acc
@@ -609,7 +609,7 @@ let uncombine l =
 	      | Cons (h1, h2), t*)
 
 
-let print ?(first="[^") ?(last="^]") ?(sep="; ") print_a out t = 
+let print ?(first="[^") ?(last="^]") ?(sep="; ") print_a out t =
   BatEnum.print ~first ~last ~sep print_a out (enum t)
 
 module Infix = struct
@@ -623,7 +623,7 @@ module Exceptionless = struct
   let rfind  = may_rfind
   let findi  = may_findi
   let rfindi = may_rfindi
-    
+
   let at list n =
     let rec aux list i =
       match (next list, i) with
