@@ -19,82 +19,82 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
 
+module S = BatSet.PSet
+module M = BatMap.PMap
 
 type ('a, 'b) t = {
-  content : ('a, 'b BatPSet.t) BatPMap.t;
+  content : ('a, 'b S.t) M.t;
   keys    : 'a -> 'a -> int     ;
   data    : 'b -> 'b -> int
 }
 
 
 let empty =
-  {content = BatPMap.empty;
+  {content = M.empty;
    keys    = compare;
    data    = compare}
 
-let is_empty t = BatPMap.is_empty t.content
+let is_empty t = M.is_empty t.content
 
-let create keys data = 
+let create keys data =
   {
-    content = BatPMap.create keys;
+    content = M.create keys;
     data    = data;
     keys    = keys
   }
 
 let find k t =
-  try BatPMap.find k t.content
-  with Not_found -> BatPSet.create t.data
+  try M.find k t.content
+  with Not_found -> S.create t.data
 
 let add k d t =
-  {(t) with content = BatPMap.add k (BatPSet.add d (find k t)) t.content}
-  
+  {(t) with content = M.add k (S.add d (find k t)) t.content}
+
 let remove_all k t =
-  {(t) with content = BatPMap.remove k t.content}
+  {(t) with content = M.remove k t.content}
 
 let remove k d t =
-  try 
-    let set = BatPSet.remove d (BatPMap.find k t.content) in
+  try
+    let set = S.remove d (M.find k t.content) in
       {(t)
        with content =
-	  if   BatPSet.is_empty set then BatPMap.remove k t.content
-	  else BatPMap.add k set t.content;
+	  if   S.is_empty set then M.remove k t.content
+	  else M.add k set t.content;
       }
   with Not_found -> t
 
-let mem k d =
-  BatPMap.mem k d.content
+let mem k d = M.mem k d.content
 
-let exists = mem
+(* let exists = mem *)
 
-let iter f d = BatPMap.iter f d.content
+let iter f d = M.iter f d.content
 
-let map  (f:('b BatPSet.t -> 'c BatPSet.t)) (cmp:('b -> 'b -> int) -> ('c -> 'c -> int)) (t:('a, 'b) t) =
-  { content = BatPMap.map f t.content;
+let map  (f:('b S.t -> 'c S.t)) (cmp:('b -> 'b -> int) -> ('c -> 'c -> int)) (t:('a, 'b) t) =
+  { content = M.map f t.content;
     keys    = t.keys;
     data    = cmp t.data}
 
-let mapi  (f:('a -> 'b BatPSet.t -> 'c BatPSet.t)) (cmp:('b -> 'b -> int) -> ('c -> 'c -> int)) (t:('a, 'b) t) =
-  { content = BatPMap.mapi f t.content;
+let mapi  (f:('a -> 'b S.t -> 'c S.t)) (cmp:('b -> 'b -> int) -> ('c -> 'c -> int)) (t:('a, 'b) t) =
+  { content = M.mapi f t.content;
     keys    = t.keys;
     data    = cmp t.data}
 
 
-let fold f d i  = BatPMap.fold f d.content i
+let fold f d i  = M.fold f d.content i
 
-let foldi f d i = BatPMap.foldi f d.content i
+let foldi f d i = M.foldi f d.content i
 
-let enum t      = BatEnum.concat (BatEnum.map (fun (k,e) -> BatEnum.map (fun x -> (k,x)) (BatPSet.enum e)) (BatPMap.enum t.content))
+let enum t      = BatEnum.concat (BatEnum.map (fun (k,e) -> BatEnum.map (fun x -> (k,x)) (S.enum e)) (M.enum t.content))
 
-let of_enum ?(keys=compare) ?(data=compare) e = 
+let of_enum ?(keys=compare) ?(data=compare) e =
   let base = create keys data in
     BatEnum.fold (fun acc (k,d) -> add k d acc) base e
 
-let print ?(first="{\n") ?(last="\n}") ?(sep=",\n") print_k print_v out t =
-  BatEnum.print ~first ~last ~sep (fun out (k, v) -> BatPrintf.fprintf out "%a: %a" print_k k print_v v) out (enum t)
+let print ?(first="{\n") ?(last="\n}") ?(sep=",\n") ?(kvsep=": ") print_k print_v out t =
+  BatEnum.print ~first ~last ~sep (fun out (k, v) -> BatPrintf.fprintf out "%a%s%a" print_k k kvsep print_v v) out (enum t)
 
 module Infix =
 struct
   let (-->) map key = find key map
   let (<--) map (key, value) = add key value map
 end
-
