@@ -1,5 +1,5 @@
 (*
- * ExtNum - Operations on arbitrary-precision numbers
+ * BatNum - Operations on arbitrary-precision numbers
  * Copyright (C) 2008 Gabriel Scherer
  *               2008 David Teller
  *
@@ -24,7 +24,7 @@
 module BaseNum = struct
   include Num
   type t = num
-  
+
   let zero       = Int 0
   let one        = Int 1
   let neg        = minus_num
@@ -36,6 +36,8 @@ module BaseNum = struct
   let modulo     = mod_num
   let pow        = power_num
   let compare    = compare_num
+  let order      = BatOrd.ord compare
+  let equal      = BatOrd.eq_comp compare
   let of_int     = num_of_int
   let to_int     = int_of_num
   let to_float   = float_of_num
@@ -52,18 +54,37 @@ module BaseNum = struct
 
 
   let of_float f =
-    let s = Printf.sprintf "%f" f in
-      try 
-	let (prefix, suffix) = BatString.split s "."    in
-	let float_digits     = String.length suffix  in
-	let divider = pow (Int 10) (Int (String.length s - float_digits)) in
-	let dividee = Big_int (Big_int.big_int_of_string  (prefix^suffix))        in
-	  div divider dividee
-      with Not_found -> of_int (BatInt.of_float f)
+    let x,e = frexp f in
+    let n,e =
+      Big_int.big_int_of_int64 (Int64.of_float (ldexp x 52)),
+      (e-52)
+    in
+    if e >= 0 then
+      Num.Big_int (Big_int.shift_left_big_int n e)
+    else
+      Num.div_num
+	(Num.Big_int n)
+	(Num.Big_int Big_int.(shift_left_big_int unit_big_int ~-e))
 end
-  
-include BatNumber.MakeNumeric(BaseNum)
-  
+
+module TaggedInfix = struct
+  let (=/), (</), (>/), (<=/), (>=/), (<>/) = Num.((=/), (</), (>/), (<=/), (>=/), (<>/))
+  let (+/), (-/), ( */ ), (//), ( **/ ) = Num.((+/), (-/), ( */ ), (//), ( **/ ))
+end
+
+module Infix = struct
+  (* infix operators without / suffix: +-*/ *)
+  include BatNumber.MakeInfix (BaseNum)
+  include TaggedInfix
+end
+
+include (BatNumber.MakeNumeric(BaseNum): BatNumber.Numeric with type t = Num.num and module Infix := Infix)
+
+(*$T of_float
+  of_float 2.5  |> to_float |> Float.approx_equal 2.5
+  of_float 1.0 |> to_float |> Float.approx_equal 1.0
+*)
+
 include Num
 let round = round_num
 let floor = floor_num
@@ -73,14 +94,5 @@ let is_integer = is_integer_num
 let approx= integer_num
 let quo   = quo_num
 let sign  = sign_num
-  
+
 let print out t = BatInnerIO.nwrite out (to_string t)
-
-module Infix = struct
-  include BatNumber.MakeInfix (BaseNum)
-  let (=/), (</), (>/), (<=/), (>=/), (<>/) = (=/), (</), (>/), (<=/), (>=/), (<>/)
-  let (+/), (-/), ( */ ), (//), ( **/ ) = (+/), (-/), ( */ ), (//), ( **/ )
-end 
-
-module Compare = BatNumber.MakeCompare (BaseNum)
-
