@@ -1,8 +1,8 @@
-(* 
+(*
  * Number - Generic interface for numbers
  * Copyright (C) 2007 Bluestorm <bluestorm dot dylc on-the-server gmail dot com>
  *               2008 David Teller
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -68,11 +68,21 @@ sig
   val ( = ) : bat__compare_t -> bat__compare_t -> bool
 end
 
+(** Idea from Shawn Wagner's mathlib *)
+module type RefOps =
+sig
+  type bat__refops_t
+  val (+=): bat__refops_t ref -> bat__refops_t -> unit
+  val (-=): bat__refops_t ref -> bat__refops_t -> unit
+  val ( *=): bat__refops_t ref -> bat__refops_t -> unit
+  val (/=): bat__refops_t ref -> bat__refops_t -> unit
+end
+
 (**
    The full set of operations of a type of numbers
 *)
 module type Numeric =
-sig  
+sig
   type t
   type discrete = t
   val zero : t
@@ -86,6 +96,9 @@ sig
   val modulo : t -> t -> t
   val pow : t -> t -> t
   val compare : t -> t -> int
+  val equal : t -> t -> bool
+  val ord : t BatOrd.ord
+
   val of_int : int -> t
   val to_int : t -> int
   val of_float: float -> t
@@ -97,9 +110,14 @@ sig
   val succ  : t -> t
   val pred  : t -> t
 
+  module Infix : Infix with type bat__infix_t = t
+  module Compare : Compare with type bat__compare_t = t
   include Infix with type bat__infix_t = t
-  include Compare with type bat__compare_t = t
+  (* Removed compare operators from base module, as they shadow
+     polymorphic ones from stdlib
 
+     include Compare with type bat__compare_t = t*)
+  include RefOps with type bat__refops_t = t
 end
 
 module type Bounded =
@@ -130,8 +148,8 @@ module type NUMERIC_BASE = sig
   val one  : t
 
 
-  (** {6 Arithmetic operations} 
-      
+  (** {6 Arithmetic operations}
+
       Depending on the implementation, some of these operations
       {i may} raise exceptions at run-time to represent over/under-flows.*)
 
@@ -159,7 +177,7 @@ module type NUMERIC_BASE = sig
 
   val of_string : string -> t
     (** Convert the representation of a number to the corresponding
-	number. Raises [Invalid_arg] if the string does not represent
+	number. @raise Invalid_argument if the string does not represent
         a valid number of type [t]*)
 
   val to_string : t -> string
@@ -176,8 +194,8 @@ module MakeInfix (Base : NUMERIC_BASE) :
   type bat__infix_t = Base.t
   let ( + ), ( - ), ( * ), ( / ), ( ** ) = Base.add, Base.sub, Base.mul, Base.div, Base.pow
   let ( -- )  x y = BatEnum.seq x Base.succ ( (>=) y )
-  let ( --- ) x y = 
-    if y >= x then x -- y 
+  let ( --- ) x y =
+    if y >= x then x -- y
     else BatEnum.seq x Base.pred ((<=) y)
 end
 
@@ -196,6 +214,16 @@ module MakeCompare (Base : NUMERIC_BASE) :
   let ( <> ) a b = Base.compare a b <> 0
 end
 
+module MakeRefOps (Base: NUMERIC_BASE) :
+  RefOps with type bat__refops_t = Base.t = struct
+
+  type bat__refops_t = Base.t
+  let (+=) a b = a := Base.add !a b
+  let (-=) a b = a := Base.sub !a b
+  let ( *=) a b = a := Base.mul !a b
+  let (/=) a b = a := Base.div !a b
+end
+
 (**
    Automated definition of operators for a given numeric type.
 
@@ -225,12 +253,18 @@ module MakeNumeric (Base : NUMERIC_BASE) : Numeric with type t = Base.t = struct
       to_float  = Base.to_float;
       of_string = Base.of_string;
       to_string = Base.to_string;
-    }    
+    }
 
   type discrete = t
 
-  include MakeInfix (Base)
-  include MakeCompare (Base)
+  let equal x y = Base.compare x y = 0
+  let ord x y = BatOrd.ord0 (Base.compare x y)
+
+  module Infix = MakeInfix (Base)
+  module Compare = MakeCompare (Base)
+  include Infix
+
+  include MakeRefOps (Base)
 end
 
 (**
