@@ -1,8 +1,8 @@
-(* 
- * ExtSet - Extended operations on sets
+(*
+ * BatSet - Extended operations on sets
  * Copyright (C) 1996 Xavier Leroy
  *               2009 David Rajchenbach-Teller, LIFO, Universite d'Orleans
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -28,7 +28,7 @@ module Concrete = struct
     | Node of 'a set * 'a * 'a set * int
 
   let empty = Empty
-    
+
   let is_empty = function Empty -> true | _ -> false
   (* Sets are represented by balanced binary trees (the heights of the
      children differ by at most 2 *)
@@ -204,15 +204,15 @@ module Concrete = struct
       Empty -> t
     | Node (l, e, r, _) -> rev_cons_iter r (C (e, l, t))
 
-  let rec enum_next l () = match !l with
+  let enum_next l () = match !l with
       E -> raise BatEnum.No_more_elements
     | C (e, s, t) -> l := cons_iter s t; e
 
-  let rec enum_backwards_next l () = match !l with
+  let enum_backwards_next l () = match !l with
       E -> raise BatEnum.No_more_elements
     | C (e, s, t) -> l := rev_cons_iter s t; e
 
-  let rec enum_count l () =
+  let enum_count l () =
     let rec aux n = function
         E -> n
       | C (e, s, t) -> aux (n + 1 + cardinal s) t
@@ -232,17 +232,17 @@ module Concrete = struct
       BatEnum.make ~next:(enum_backwards_next l) ~count:(enum_count l) ~clone
     in make (rev_cons_iter t E)
 
-  let of_enum cmp e = 
+  let of_enum cmp e =
     BatEnum.fold (fun acc elem -> add cmp elem acc) empty e
 
-  let print ?(first="{\n") ?(last="\n}") ?(sep=",\n") print_elt out t =
+  let print ?(first="{") ?(last="}") ?(sep=",") print_elt out t =
     BatEnum.print ~first ~last ~sep (fun out e -> BatPrintf.fprintf out "%a" print_elt e) out (enum t)
 
   let filter cmp f e = fold (fun x acc -> if f x then add cmp x acc else acc) e empty
 
   let filter_map cmp f e = fold (fun x acc -> match f x with Some v -> add cmp v acc | _ -> acc) e empty
 
-  let choose = min_elt
+  let choose = min_elt (* I'd rather this chose the root, but okay *)
 
   let rec for_all p = function
       Empty -> true
@@ -281,6 +281,17 @@ module Concrete = struct
               join (union cmp12 l1 l2) v2 (union cmp12 r1 r2)
             end
 
+  let rec sym_diff cmp12 s1 s2 =
+    match (s1, s2) with
+        (Empty, t2) -> t2
+      | (t1, Empty) -> t1
+      | (Node(l1, v1, r1, _), t2) ->
+          match split cmp12 v1 t2 with
+              (l2, false, r2) ->
+		join (sym_diff cmp12 l1 l2) v1 (sym_diff cmp12 r1 r2)
+            | (l2, true, r2) ->
+		concat (sym_diff cmp12 l1 l2) (sym_diff cmp12 r1 r2)
+
   let rec inter cmp12 s1 s2 =
     match (s1, s2) with
         (Empty, t2) -> Empty
@@ -303,6 +314,32 @@ module Concrete = struct
             | (l2, true, r2) ->
                 concat (diff cmp12 l1 l2) (diff cmp12 r1 r2)
 
+  let rec disjoint cmp12 s1 s2 =
+    match (s1, s2) with
+        (Empty, _)
+      | (_, Empty) -> true
+      | (Node(l1, v1, r1, _), t2) ->
+          match split cmp12 v1 t2 with
+              (l2, false, r2) ->
+                disjoint cmp12 l1 l2 && disjoint cmp12 r1 r2
+            | (l2, true, r2) -> false
+
+  let compare cmp s1 s2 =
+    let rec compare_aux t1' t2' =
+      match (t1', t2') with
+          E, E ->  0
+        | E, _ -> -1
+        | _, E ->  1
+        | C (e1, r1, t1), C (e2, r2, t2) ->
+            let c = cmp e1 e2 in
+            if c = 0 then
+              compare_aux (cons_iter r1 t1) (cons_iter r2 t2)
+            else
+              c in
+    compare_aux (cons_iter s1 E) (cons_iter s2 E)
+
+  let equal cmp s1 s2 = compare cmp s1 s2 = 0
+
   let rec subset cmp s1 s2 =
     match (s1, s2) with
         Empty, _ ->
@@ -322,90 +359,49 @@ end
 module type S =
 sig
   type elt
-
   type t
-
   val empty: t
-
   val is_empty: t -> bool
-
   val singleton: elt -> t
-
   val mem: elt -> t -> bool
-
   val add: elt -> t -> t
-
   val remove: elt -> t -> t
-
   val union: t -> t -> t
-
   val inter: t -> t -> t
-
   val diff: t -> t -> t
-
+  val sym_diff: t -> t -> t
   val compare: t -> t -> int
-
   val equal: t -> t -> bool
-
   val subset: t -> t -> bool
-
+  val disjoint: t -> t -> bool
   val compare_subset: t -> t -> int
-
   val iter: (elt -> unit) -> t -> unit
-
   val map: (elt -> elt) -> t -> t
-
   val filter: (elt -> bool) -> t -> t
-
   val filter_map: (elt -> elt option) -> t -> t
-
   val fold: (elt -> 'a -> 'a) -> t -> 'a -> 'a
-
   val for_all: (elt -> bool) -> t -> bool
-
   val exists: (elt -> bool) -> t -> bool
-
   val partition: (elt -> bool) -> t -> t * t
-
   val split: elt -> t -> t * bool * t
-
   val cardinal: t -> int
-
   val elements: t -> elt list
-
   val min_elt: t -> elt
-
   val max_elt: t -> elt
-
   val choose: t -> elt
-
   val pop: t -> elt * t
-
   val enum: t -> elt BatEnum.t
-
   val backwards: t -> elt BatEnum.t
-
   val of_enum: elt BatEnum.t -> t
-
-
-  (** {6 Boilerplate code}*)
-
-  (** {7 Printing}*)
-    
-  val print :  ?first:string -> ?last:string -> ?sep:string -> 
-    ('a BatInnerIO.output -> elt -> unit) -> 
+  val print :  ?first:string -> ?last:string -> ?sep:string ->
+    ('a BatInnerIO.output -> elt -> unit) ->
     'a BatInnerIO.output -> t -> unit
-
-  (** {6 Override modules}*)
-    
   (** Operations on {!Set} without exceptions.*)
   module Exceptionless : sig
     val min_elt: t -> elt option
     val max_elt: t -> elt option
     val choose:  t -> elt option
   end
-    
-    
   (** Operations on {!Set} with labels. *)
   module Labels : sig
     val iter : f:(elt -> unit) -> t -> unit
@@ -417,11 +413,11 @@ sig
     val filter_map: f:(elt -> elt option) -> t -> t
     val partition : f:(elt -> bool) -> t -> t * t
   end
-    
+
 end
     (** Output signature of the functor {!Set.Make}. *)
 
-module Make (Ord : OrderedType) = 
+module Make (Ord : OrderedType) =
 struct
   include Set.Make(Ord)
 
@@ -431,8 +427,6 @@ struct
   external impl_of_t : t -> implementation = "%identity"
   external t_of_impl : implementation -> t = "%identity"
 
-  type iter = E | C of elt * implementation * iter
-
   let cardinal t = Concrete.cardinal (impl_of_t t)
   let enum t = Concrete.enum (impl_of_t t)
   let of_enum e = t_of_impl (Concrete.of_enum Ord.compare e)
@@ -466,12 +460,16 @@ struct
 
   let singleton e = t_of_impl (Concrete.singleton e)
   let elements t = Concrete.elements (impl_of_t t)
-    
+
   let union s1 s2 = t_of_impl (Concrete.union Ord.compare (impl_of_t s1) (impl_of_t s2))
   let diff s1 s2 = t_of_impl (Concrete.diff Ord.compare (impl_of_t s1) (impl_of_t s2))
   let inter s1 s2 = t_of_impl (Concrete.inter Ord.compare (impl_of_t s1) (impl_of_t s2))
+  let sym_diff s1 s2 = t_of_impl (Concrete.sym_diff Ord.compare (impl_of_t s1) (impl_of_t s2))
 
+  let compare t1 t2 = Concrete.compare Ord.compare (impl_of_t t1) (impl_of_t t2)
+  let equal t1 t2 = Concrete.equal Ord.compare (impl_of_t t1) (impl_of_t t2)
   let subset t1 t2 = Concrete.subset Ord.compare (impl_of_t t1) (impl_of_t t2)
+  let disjoint t1 t2 = Concrete.disjoint Ord.compare (impl_of_t t1) (impl_of_t t2)
 
   let rec compare_subset s1 s2 =
     match (s1, impl_of_t s2) with
@@ -515,124 +513,172 @@ struct
   end
 end
 
+module PSet = struct (*$< PSet *)
 
-module StringSet  = Make(String)
-module IStringSet = Make(BatString.IString)
-module NumStringSet = Make(BatString.NumString)
-module RopeSet    = Make(BatRope)
-module IRopeSet   = Make(BatRope.IRope)
-module IntSet     = Make(BatInt)
-module CharSet    = Make(Char)
+  type 'a t = {
+    cmp : 'a -> 'a -> int;
+    set : 'a Concrete.set;
+  }
 
-(*
- * PMap - Polymorphic sets
- * Copyright (C) 1996-2003 Xavier Leroy, Nicolas Cannasse, Markus Mottl
- * Copyright (C)      2008 David Teller
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version,
- * with the special exception on linking described in file LICENSE.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *)
+  type 'a enumerable = 'a t
+  type 'a mappable = 'a t
+
+  let empty    = { cmp = compare; set = Concrete.empty }
+  let create cmp  = { cmp = cmp; set = Concrete.empty }
+  let get_cmp {cmp} = cmp
+
+(*$T get_cmp
+  get_cmp (create Int.compare) == Int.compare
+*)
 
 
-type 'a t = {
-  cmp : 'a -> 'a -> int;
-  set : 'a Concrete.set;
-}
+  let singleton ?(cmp = compare) x = { cmp = cmp; set = Concrete.singleton x }
+  let is_empty s = Concrete.is_empty s.set
+  let mem x s = Concrete.mem s.cmp x s.set
+  let add x s  = { s with set = Concrete.add s.cmp x s.set }
+  let remove x s = { s with set = Concrete.remove s.cmp x s.set }
+  let iter f s = Concrete.iter f s.set
+  let fold f s acc = Concrete.fold f s.set acc
+  let map f s =
+    { cmp = Pervasives.compare; set = Concrete.map Pervasives.compare f s.set }
+  let filter f s = { s with set = Concrete.filter s.cmp f s.set }
+  let filter_map f s =
+    { cmp = compare; set = Concrete.filter_map compare f s.set }
+  let exists f s = Concrete.exists f s.set
+  let cardinal s = fold (fun _ acc -> acc + 1) s 0
+  let elements s = Concrete.elements s.set
+  let choose s = Concrete.choose s.set
+  let min_elt s = Concrete.min_elt s.set
+  let max_elt s = Concrete.max_elt s.set
+  let enum s = Concrete.enum s.set
+  let of_enum e = { cmp = compare; set = Concrete.of_enum compare e }
+  let of_enum_cmp ~cmp t = { cmp = cmp; set = Concrete.of_enum cmp t }
+  let of_list l = List.fold_left (fun a x -> add x a) empty l
+  let print ?first ?last ?sep print_elt out s =
+    Concrete.print ?first ?last ?sep print_elt out s.set
+  let for_all f s = Concrete.for_all f s.set
+  let partition f s =
+    let l, r = Concrete.partition s.cmp f s.set in
+    { s with set = l }, { s with set = r }
+  let pop s =
+    let v, s' = Concrete.pop s.set in
+    v, { s with set = s' }
+  let split e s =
+    let s1, found, s2 = Concrete.split s.cmp e s.set in
+    { s with set = s1 }, found, { s with set = s2 }
+  let union s1 s2 =
+    { s1 with set = Concrete.union s1.cmp s1.set s2.set }
+  let diff s1 s2 =
+    { s1 with set = Concrete.diff s1.cmp s1.set s2.set }
+  let sym_diff s1 s2 =
+    { s1 with set = Concrete.sym_diff s1.cmp s1.set s2.set }
+  let intersect s1 s2 =
+    { s1 with set = Concrete.inter s1.cmp s1.set s2.set }
+  let compare s1 s2 = Concrete.compare s1.cmp s1.set s2.set
+  let equal s1 s2 = Concrete.equal s1.cmp s1.set s2.set
+  let subset s1 s2 = Concrete.subset s1.cmp s1.set s2.set
+  let disjoint s1 s2 = Concrete.disjoint s1.cmp s1.set s2.set
+end (*$>*)
+
+type 'a t = 'a Concrete.set
 
 type 'a enumerable = 'a t
 type 'a mappable = 'a t
 
-let empty    = { cmp = compare; set = Concrete.empty }
+let empty    = Concrete.empty
 
-let create cmp  = { cmp = cmp; set = Concrete.empty }
+let singleton x = Concrete.singleton x
 
-let singleton ?(cmp = compare) x = { cmp = cmp; set = Concrete.singleton x }
+let is_empty s = s = Concrete.Empty
 
-let is_empty s = s.set = Concrete.Empty
+let mem x s = Concrete.mem Pervasives.compare x s
 
-let mem x s = Concrete.mem s.cmp x s.set
+let add x s  = Concrete.add Pervasives.compare x s
 
-let add x s  = { s with set = Concrete.add s.cmp x s.set }
+let remove x s = Concrete.remove Pervasives.compare x s
 
-let remove x s = { s with set = Concrete.remove s.cmp x s.set }
+let iter f s = Concrete.iter f s
 
-let iter f s = Concrete.iter f s.set
+let fold f s acc = Concrete.fold f s acc
 
-let fold f s acc = Concrete.fold f s.set acc
+let map f s = Concrete.map Pervasives.compare f s
 
-let map f s =
-  { cmp = compare; set = Concrete.map s.cmp f s.set }
+(*$T map
+  map (fun _x -> 1) (of_list [1;2;3]) |> cardinal = 1
+*)
 
-let filter f s = { s with set = Concrete.filter s.cmp f s.set }
+let filter f s = Concrete.filter Pervasives.compare f s
 
-let filter_map f s =
-  { cmp = compare; set = Concrete.filter_map compare f s.set }
+let filter_map f s = Concrete.filter_map Pervasives.compare f s
 
-let exists f s = Concrete.exists f s.set
+let exists f s = Concrete.exists f s
 
-let cardinal s =
-  fold (fun _ acc -> acc + 1) s 0
+let cardinal s = fold (fun _ acc -> acc + 1) s 0
 
-let choose s = Concrete.choose s.set
+let elements s = Concrete.elements s
 
-let min_elt s = Concrete.min_elt s.set
+let choose s = Concrete.choose s
 
-let max_elt s = Concrete.max_elt s.set
+let min_elt s = Concrete.min_elt s
 
-let enum s = Concrete.enum s.set
+(*$Q min_elt
+  (Q.list Q.small_int) (fun l -> l = [] || \
+    let xs = List.map (fun i -> i mod 2, i) l in \
+    let s = ref (of_list xs) in \
+    let m = ref (min_elt !s) in \
+    while fst !m = 0 do \
+      s := remove !m !s; \
+      s := add (2,snd !m) !s; \
+      m := min_elt !s; \
+    done; \
+    for_all (fun (x,_) -> x <> 0) !s \
+  )
+*)
 
-let of_enum e =
-  { cmp = compare; set = Concrete.of_enum compare e }
+let max_elt s = Concrete.max_elt s
 
-let of_enum_cmp ~cmp t =
-  { cmp = cmp; set = Concrete.of_enum cmp t }
+let enum s = Concrete.enum s
+
+let of_enum e = Concrete.of_enum Pervasives.compare e
+
+let backwards s = Concrete.backwards s
 
 let of_list l = List.fold_left (fun a x -> add x a) empty l
 
+(*$Q of_list
+  (Q.list Q.small_int) (fun l -> let xs = List.map (fun i -> i mod 5, i) l in \
+    let s1 = of_list xs |> enum |> List.of_enum in \
+    let s2 = List.sort_unique Pervasives.compare xs in \
+    s1 = s2 \
+  )
+*)
+
 let print ?first ?last ?sep print_elt out s =
-  Concrete.print ?first ?last ?sep print_elt out s.set
+  Concrete.print ?first ?last ?sep print_elt out s
 
-let for_all f s = Concrete.for_all f s.set
+let for_all f s = Concrete.for_all f s
+let partition f s = Concrete.partition Pervasives.compare f s
+let pop s = Concrete.pop s
+let split e s = Concrete.split Pervasives.compare e s
+let union s1 s2 = Concrete.union Pervasives.compare s1 s2
+let diff s1 s2 = Concrete.diff Pervasives.compare s1 s2
+let sym_diff s1 s2 = Concrete.sym_diff Pervasives.compare s1 s2
+let intersect s1 s2 = Concrete.inter Pervasives.compare s1 s2
+let compare s1 s2 = Concrete.compare Pervasives.compare s1 s2
+let equal s1 s2 = Concrete.equal Pervasives.compare s1 s2
+let subset s1 s2 = Concrete.subset Pervasives.compare s1 s2
+let disjoint s1 s2 = Concrete.disjoint Pervasives.compare s1 s2
 
-let partition f s =
-  let l, r = Concrete.partition s.cmp f s.set in
-  { s with set = l }, { s with set = r }
-
-let filter f s = { s with set = Concrete.filter s.cmp f s.set }
-
-let pop s =
-  let v, s' = Concrete.pop s.set in
-  v, { s with set = s' }
-
-let split e s =
-  let s1, found, s2 = Concrete.split s.cmp e s.set in
-  { s with set = s1 }, found, { s with set = s2 }
-
-let union s1 s2 =
-  { s1 with set = Concrete.union s1.cmp s1.set s2.set }
-
-let diff s1 s2 =
-  { s1 with set = Concrete.diff s1.cmp s1.set s2.set }
-
-let intersect s1 s2 =
-  { s1 with set = Concrete.inter s1.cmp s1.set s2.set }
-
-let subset s1 s2 = Concrete.subset s1.cmp s1.set s2.set
-
-(**T subset
+(*$T subset
    subset (of_list [1;2;3]) (of_list [1;2;3;4])
    not (subset (of_list [1;2;3;5]) (of_list [1;2;3;4]))
    not (subset (of_list [1;2;3;4]) (of_list [1;2;3]))
-**)
+ *)
+
+(*$T compare
+  compare (of_list [1;2;3]) (of_list [1;2;3;4]) <> 0
+  let a = of_list [1;2;3] and b = of_list [1;2;3;4] in compare a b = - (compare b a)
+  let a = of_list [1;2;3] and b = of_list [1;2;3;4] and c = of_list [3;1;2] in\
+    compare a b = - (compare b c)
+  compare (of_list [1;2;3]) (of_list [3;1;2]) = 0
+ *)
