@@ -1,7 +1,7 @@
 (*
  * re-implementation of SML's Substring library in OCaml.
  * Copyright (C) 2008 Edgar Friendly <thelema314@gmail.com>
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -36,7 +36,7 @@ let create len = String.make len '\000', 0, len
 (*
 let of_chan chan =
   let tempsize = 16384 in
-  let buf = Buffer.create tempsize 
+  let buf = Buffer.create tempsize
   and tmp = String.create tempsize in
   let n = ref 0 in
   while n := input chan tmp 0 tempsize; !n > 0 do
@@ -47,15 +47,15 @@ let of_chan chan =
 
 let of_input inp =
   let tempsize = 16384 in
-  let buf = Buffer.create tempsize 
+  let buf = Buffer.create tempsize
   and tmp = String.create tempsize in
   let n = ref 0 in
   while n := BatIO.input inp tmp 0 tempsize; !n > 0 do
     Buffer.add_substring buf tmp 0 !n;
   done;
   Buffer.contents buf, 0, Buffer.length buf
-    
-let substring str off len = 
+
+let substring str off len =
   let sl = String.length str in
   if off < 0 then invalid_arg "Substring.substring: negative offset not allowed";
   if len < 0 then invalid_arg "Substring.substring: negative length not allowed";
@@ -72,20 +72,20 @@ let base s = s
 
 let is_empty (_,_,len) = len = 0
 
-let getc (str,off,len) = 
+let getc (str,off,len) =
   if len = 0 then None else Some (str.[off], (str, off+1, len-1))
 
 let first (str,off,len) = if len = 0 then None else Some str.[off]
 
-let triml k (str,off,len) = 
+let triml k (str,off,len) =
   if k < 0 then invalid_arg "Substring.triml: negative trim not allowed";
   if k > len then (str, off+len, 0) else (str, off+k, len-k)
 
-let trimr k (str,off,len) = 
+let trimr k (str,off,len) =
   if k < 0 then invalid_arg "Substring.triml: negative trim not allowed";
   if k > len then (str, off, 0) else (str, off, len-k)
 
-let get k (str, off, len) = 
+let get (str, off, len) k =
   if k < 0 then invalid_arg "Substring.get: negative index not allowed";
   if k > len then invalid_arg "Substring.get: index outside of substring";
   str.[off+k]
@@ -93,23 +93,23 @@ let get k (str, off, len) =
 let size (_,_,len) = len
 let length = size
 
-let slice (str,off,len) off2 len2_opt = 
+let slice (str,off,len) off2 len2_opt =
   if off2 < 0 then invalid_arg "Substring.slice: negative offset not allowed";
   let len2 = match len2_opt with None -> len-off2 | Some i -> i in
   if len2 + off2 > len then invalid_arg "Substring.slice: invalid slice";
   (str, off+off2, len2)
 
-let concat ssl = 
+let concat ssl =
   let len = List.fold_left (fun acc (_,_,l) ->acc+l) 0 ssl in
   let item = String.create len in
-  let write = 
+  let write =
     let pos = ref 0 in
     fun (s,o,len) -> String.unsafe_blit s o item !pos len; pos := !pos + len
   in
   List.iter write ssl;
   item
 
-let explode (str,off,len) = 
+let explode (str,off,len) =
   let rec exp i l =
     if i < off then l else exp (i - 1) (str.[i] :: l) in
   exp (off+len) []
@@ -117,32 +117,55 @@ let explode (str,off,len) =
 let is_prefix str1 (str2, off, len) =
   let l1 = String.length str1 in
   if l1 > len then false
-  else 
-    let rec loop i = 
-      if i < 0 then true 
+  else
+    let rec loop i =
+      if i < 0 then true
       else if str1.[i] <> str2.[off+i] then false
       else loop (i-1) in
-    loop l1
+    loop (pred l1)
 
 let compare (str1, off1, len1) (str2, off2, len2) =
-  let rec loop i = 
-    if i > len1 then if i > len2 then 0 else 1
-    else if i > len2 then -1 
-    else 
+  let rec loop i =
+    if i >= len1 then if i >= len2 then 0 else -1
+    else if i >= len2 then 1
+    else
       let c1 = str1.[off1+i] and c2 = str2.[off2+i] in
-      if c1 > c2 then -1 
-      else if c1 < c2 then 1 
+      if c1 > c2 then 1
+      else if c1 < c2 then -1
       else loop (i+1)
-  in 
+  in
   loop 0
 
+let index_from (str, off, len) i c = 
+  let rec aux k = 
+    if k = len then raise Not_found
+    else if str.[off+k] = c then k
+    else aux (k+1)
+  in 
+  if i > len || i < 0 then invalid_arg "Substring.index_from" 
+  else aux i
+
+let index sus c = index_from sus 0 c
+
+let rindex_from (str, off, len) i c = 
+  let rec aux k = 
+    if k < 0 then raise Not_found
+    else if str.[off+k] = c then k
+    else aux (k-1)
+  in 
+  if i > len || i < 0 then invalid_arg "Substring.rindex_from" 
+  else aux i
+
+let rindex sus c = rindex_from sus (size sus - 1) c
+
+
 (** not implemented: collate *)
 
 let dropl p (str,off,len) =
   let i = ref 0 in
   while !i < len && p str.[off+ !i] do incr i; done;
   (str, off+ !i, len- !i)
-  
+
 let dropr p (str, off, len) =
   let i = ref len in
   while !i > 0 && p str.[off+ !i - 1] do decr i; done;
@@ -158,11 +181,11 @@ let taker p (str, off, len) =
   while !i > 0 && p str.[off+ !i - 1] do decr i; done;
   (str, off+ !i, len- !i)
 
-let splitl p (str, off, len) = 
+let splitl p (str, off, len) =
   let i = ref 0 in
   while !i < len && p str.[off+ !i] do incr i; done;
   (str, off, !i), (str, off+ !i, len- !i)
-  
+
 let splitr p (str, off, len) =
   let i = ref len in
   while !i > 0 && p str.[off+ !i - 1] do decr i; done;
@@ -175,10 +198,10 @@ let split_at k (str, off, len) =
 
 (** not implemented: position *)
 
-let span (str1, off1, len1) (str2, off2, len2) =
-  if str1 != str2 then invalid_arg 
+let span (str1, off1, _len1) (str2, off2, len2) =
+  if str1 != str2 then invalid_arg
     "Substring.span: must be substrings of same parent";
-  if off1 > off2 + len2 then invalid_arg 
+  if off1 > off2 + len2 then invalid_arg
     "Substring.span: first substring must not be to the right of the second";
   (str1, off1, off2+len2-off1)
 
@@ -201,7 +224,7 @@ let fields p (str, off, len) =
   while !j < len do
     while !j < len && not (p str.[off+ !j]) do incr j; done;
     BatRefList.push acc (str, !i, !j);
-    incr j; i := !j; 
+    incr j; i := !j;
   done;
   BatRefList.to_list acc
 
@@ -226,10 +249,10 @@ let iter f (str, off, len) =
 
 let trim x = dropl BatChar.is_whitespace (dropr BatChar.is_whitespace x)
 
-let split_on_char c (str, off, len) = 
+let split_on_char c (str, off, len) =
   let rec loop acc last_pos pos =
-    if pos = -1 then
-      (str, 0, last_pos) :: acc
+    if pos = off - 1 then
+      (str, off, last_pos - off) :: acc
     else
       if str.[pos] = c then
         let pos1 = pos + 1 in
@@ -237,11 +260,11 @@ let split_on_char c (str, off, len) =
         loop (sub_str :: acc) pos (pos - 1)
       else loop acc last_pos (pos - 1)
   in
-  loop [] len (len - 1)
-  
+  loop [] (off+len) (off + len - 1)
+
 let split_on_pipe str = split_on_char '|' str;;
 let split_on_dot str = split_on_char '.' str;;
 let split_on_comma str = split_on_char ',' str;;
 let split_on_slash str = split_on_char '/' str;;
 
-let print oc ss = iter (fun c -> BatIO.write oc c) ss 
+let print oc ss = iter (fun c -> BatIO.write oc c) ss
