@@ -1,6 +1,6 @@
 open OUnit
 open BatBigarray
-open BatStd
+open BatPervasives
 
 let array     = [|'1';'2';'3';'4';'5'|]
 let array2    = [|[|'1';'2';'3';'4';'5'|];
@@ -18,8 +18,7 @@ let bigarray1 = Array1.of_array char c_layout array
 let bigarray2 = Array2.of_array char c_layout array2
 let bigarray3 = Array3.of_array char c_layout array3
 
-let utf8   = BatUTF8.of_string string
-let rope   = BatRope.of_ustring utf8
+let text   = BatText.of_string string
 
 module C =
 struct
@@ -70,14 +69,15 @@ let test_map_enums () =
     let aeq = assert_equal
                 ~cmp:(fun m1 m2 -> M.compare (fun _ _ -> 0) m1 m2 = 0)
                 ~printer:(BatPrintf.sprintf2 "%a"
-                            (print BatChar.print (fun io v -> ())))
+                            (print BatChar.print (fun _io _v -> ())))
     in
       aeq (of_enum (enum source)) (of_enum (backwards source));
       aeq source (of_enum (backwards source))
 
-open BatRope
+(*
+open Ulib.Text
 let test_rope_enums () =
-    let source = rope in
+    let source = text in
     let aeq = assert_equal ~printer:(BatPrintf.sprintf2 "%a" print) in
       aeq (of_backwards (enum source)) (of_enum (backwards source));
       aeq source (of_backwards (backwards source));
@@ -88,6 +88,7 @@ let test_UTF8_enums () =
     let aeq = assert_equal ~printer:(BatPrintf.sprintf2 "%a" print) in
       aeq (of_backwards (enum source)) (of_enum (backwards source));
       aeq source (of_backwards (backwards source));
+*)
 
 open BatArray
 let test_bigarray_enums () =
@@ -115,13 +116,52 @@ let test_uncombine () =
   aeq a c;
   aeq b d
 
+(* BatEnum.from should not call the user function after No_more_elements was raised. *)
+let test_from () =
+  let nb_calls = ref 0 in
+  let next () =
+  	incr nb_calls ;
+    if !nb_calls <= 5 then !nb_calls
+    else raise BatEnum.No_more_elements in
+  let e = BatEnum.merge (fun _ _ -> true) (BatEnum.from next) (1 -- 5) in
+  let nb_res = BatEnum.hard_count e in
+  assert_equal ~printer:string_of_int 10 nb_res ;
+  assert_equal ~printer:string_of_int 6 !nb_calls
+
+(* Same as above for BatEnum.from_while *)
+let test_from_while () =
+  let nb_calls = ref 0 in
+  let next () =
+    incr nb_calls ;
+    if !nb_calls <= 5 then Some !nb_calls
+    else None in
+  let e = BatEnum.merge (fun _ _ -> true) (BatEnum.from_while next) (1 -- 5) in
+  let nb_res = BatEnum.hard_count e in
+  assert_equal ~printer:string_of_int 10 nb_res ;
+  assert_equal ~printer:string_of_int 6 !nb_calls
+
+(* Same as above for BatEnum.from_loop *)
+let test_from_loop () =
+  let nb_calls = ref 0 in
+  let next prev =
+    incr nb_calls ;
+    if !nb_calls <= 5 then prev+1, !nb_calls
+    else raise BatEnum.No_more_elements in
+  let e = BatEnum.merge (fun _ _ -> true) (BatEnum.from_loop 0 next) (1 -- 5) in
+  let nb_res = BatEnum.hard_count e in
+  assert_equal ~printer:string_of_int 10 nb_res ;
+  assert_equal ~printer:string_of_int 6 !nb_calls
+
 let tests = "BatEnum" >::: [
   "Array" >:: test_array_enums;
   "List" >:: test_list_enums;
   "String" >:: test_string_enums;
-  "Rope" >:: test_rope_enums;
-  "UTF8" >:: test_UTF8_enums;
+(*  "Rope" >:: test_rope_enums;
+  "UTF8" >:: test_UTF8_enums; *)
   "bigarray" >:: test_bigarray_enums;
   "Set" >:: test_set_enums;
   "uncombine" >:: test_uncombine;
+  "from" >:: test_from;
+  "from_while" >:: test_from_while;
+  "from_loop" >:: test_from_loop;
 ]
